# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workitems_single_get_response_data_relationships_approvals import (
        WorkitemsSingleGetResponseDataRelationshipsApprovals,
    )
    from ..models.workitems_single_get_response_data_relationships_assignee import (
        WorkitemsSingleGetResponseDataRelationshipsAssignee,
    )
    from ..models.workitems_single_get_response_data_relationships_attachments import (
        WorkitemsSingleGetResponseDataRelationshipsAttachments,
    )
    from ..models.workitems_single_get_response_data_relationships_author import (
        WorkitemsSingleGetResponseDataRelationshipsAuthor,
    )
    from ..models.workitems_single_get_response_data_relationships_backlinked_work_items import (
        WorkitemsSingleGetResponseDataRelationshipsBacklinkedWorkItems,
    )
    from ..models.workitems_single_get_response_data_relationships_categories import (
        WorkitemsSingleGetResponseDataRelationshipsCategories,
    )
    from ..models.workitems_single_get_response_data_relationships_comments import (
        WorkitemsSingleGetResponseDataRelationshipsComments,
    )
    from ..models.workitems_single_get_response_data_relationships_externally_linked_work_items import (
        WorkitemsSingleGetResponseDataRelationshipsExternallyLinkedWorkItems,
    )
    from ..models.workitems_single_get_response_data_relationships_linked_oslc_resources import (
        WorkitemsSingleGetResponseDataRelationshipsLinkedOslcResources,
    )
    from ..models.workitems_single_get_response_data_relationships_linked_revisions import (
        WorkitemsSingleGetResponseDataRelationshipsLinkedRevisions,
    )
    from ..models.workitems_single_get_response_data_relationships_linked_work_items import (
        WorkitemsSingleGetResponseDataRelationshipsLinkedWorkItems,
    )
    from ..models.workitems_single_get_response_data_relationships_module import (
        WorkitemsSingleGetResponseDataRelationshipsModule,
    )
    from ..models.workitems_single_get_response_data_relationships_planned_in import (
        WorkitemsSingleGetResponseDataRelationshipsPlannedIn,
    )
    from ..models.workitems_single_get_response_data_relationships_project import (
        WorkitemsSingleGetResponseDataRelationshipsProject,
    )
    from ..models.workitems_single_get_response_data_relationships_test_steps import (
        WorkitemsSingleGetResponseDataRelationshipsTestSteps,
    )
    from ..models.workitems_single_get_response_data_relationships_votes import (
        WorkitemsSingleGetResponseDataRelationshipsVotes,
    )
    from ..models.workitems_single_get_response_data_relationships_watches import (
        WorkitemsSingleGetResponseDataRelationshipsWatches,
    )
    from ..models.workitems_single_get_response_data_relationships_work_records import (
        WorkitemsSingleGetResponseDataRelationshipsWorkRecords,
    )


T = TypeVar("T", bound="WorkitemsSingleGetResponseDataRelationships")


@_attrs_define
class WorkitemsSingleGetResponseDataRelationships:
    """
    Attributes:
        approvals (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsApprovals]):
        assignee (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsAssignee]):
        attachments (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsAttachments]):
        author (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsAuthor]):
        backlinked_work_items (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsBacklinkedWorkItems]):
        categories (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsCategories]):
        comments (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsComments]):
        externally_linked_work_items (Union[Unset,
            WorkitemsSingleGetResponseDataRelationshipsExternallyLinkedWorkItems]):
        linked_oslc_resources (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsLinkedOslcResources]):
        linked_revisions (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsLinkedRevisions]):
        linked_work_items (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsLinkedWorkItems]):
        module (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsModule]):
        planned_in (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsPlannedIn]):
        project (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsProject]):
        test_steps (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsTestSteps]):
        votes (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsVotes]):
        watches (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsWatches]):
        work_records (Union[Unset, WorkitemsSingleGetResponseDataRelationshipsWorkRecords]):
    """

    approvals: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsApprovals"
    ] = UNSET
    assignee: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsAssignee"
    ] = UNSET
    attachments: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsAttachments"
    ] = UNSET
    author: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsAuthor"
    ] = UNSET
    backlinked_work_items: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsBacklinkedWorkItems"
    ] = UNSET
    categories: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsCategories"
    ] = UNSET
    comments: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsComments"
    ] = UNSET
    externally_linked_work_items: Union[
        Unset,
        "WorkitemsSingleGetResponseDataRelationshipsExternallyLinkedWorkItems",
    ] = UNSET
    linked_oslc_resources: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsLinkedOslcResources"
    ] = UNSET
    linked_revisions: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsLinkedRevisions"
    ] = UNSET
    linked_work_items: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsLinkedWorkItems"
    ] = UNSET
    module: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsModule"
    ] = UNSET
    planned_in: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsPlannedIn"
    ] = UNSET
    project: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsProject"
    ] = UNSET
    test_steps: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsTestSteps"
    ] = UNSET
    votes: Union[Unset, "WorkitemsSingleGetResponseDataRelationshipsVotes"] = (
        UNSET
    )
    watches: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsWatches"
    ] = UNSET
    work_records: Union[
        Unset, "WorkitemsSingleGetResponseDataRelationshipsWorkRecords"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        approvals: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.approvals, Unset):
            approvals = self.approvals.to_dict()

        assignee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assignee, Unset):
            assignee = self.assignee.to_dict()

        attachments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = self.attachments.to_dict()

        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        backlinked_work_items: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.backlinked_work_items, Unset):
            backlinked_work_items = self.backlinked_work_items.to_dict()

        categories: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.categories, Unset):
            categories = self.categories.to_dict()

        comments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.comments, Unset):
            comments = self.comments.to_dict()

        externally_linked_work_items: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.externally_linked_work_items, Unset):
            externally_linked_work_items = (
                self.externally_linked_work_items.to_dict()
            )

        linked_oslc_resources: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.linked_oslc_resources, Unset):
            linked_oslc_resources = self.linked_oslc_resources.to_dict()

        linked_revisions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.linked_revisions, Unset):
            linked_revisions = self.linked_revisions.to_dict()

        linked_work_items: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.linked_work_items, Unset):
            linked_work_items = self.linked_work_items.to_dict()

        module: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.module, Unset):
            module = self.module.to_dict()

        planned_in: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.planned_in, Unset):
            planned_in = self.planned_in.to_dict()

        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        test_steps: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.test_steps, Unset):
            test_steps = self.test_steps.to_dict()

        votes: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.votes, Unset):
            votes = self.votes.to_dict()

        watches: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.watches, Unset):
            watches = self.watches.to_dict()

        work_records: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.work_records, Unset):
            work_records = self.work_records.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if approvals is not UNSET:
            field_dict["approvals"] = approvals
        if assignee is not UNSET:
            field_dict["assignee"] = assignee
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if author is not UNSET:
            field_dict["author"] = author
        if backlinked_work_items is not UNSET:
            field_dict["backlinkedWorkItems"] = backlinked_work_items
        if categories is not UNSET:
            field_dict["categories"] = categories
        if comments is not UNSET:
            field_dict["comments"] = comments
        if externally_linked_work_items is not UNSET:
            field_dict["externallyLinkedWorkItems"] = (
                externally_linked_work_items
            )
        if linked_oslc_resources is not UNSET:
            field_dict["linkedOslcResources"] = linked_oslc_resources
        if linked_revisions is not UNSET:
            field_dict["linkedRevisions"] = linked_revisions
        if linked_work_items is not UNSET:
            field_dict["linkedWorkItems"] = linked_work_items
        if module is not UNSET:
            field_dict["module"] = module
        if planned_in is not UNSET:
            field_dict["plannedIn"] = planned_in
        if project is not UNSET:
            field_dict["project"] = project
        if test_steps is not UNSET:
            field_dict["testSteps"] = test_steps
        if votes is not UNSET:
            field_dict["votes"] = votes
        if watches is not UNSET:
            field_dict["watches"] = watches
        if work_records is not UNSET:
            field_dict["workRecords"] = work_records

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workitems_single_get_response_data_relationships_approvals import (
            WorkitemsSingleGetResponseDataRelationshipsApprovals,
        )
        from ..models.workitems_single_get_response_data_relationships_assignee import (
            WorkitemsSingleGetResponseDataRelationshipsAssignee,
        )
        from ..models.workitems_single_get_response_data_relationships_attachments import (
            WorkitemsSingleGetResponseDataRelationshipsAttachments,
        )
        from ..models.workitems_single_get_response_data_relationships_author import (
            WorkitemsSingleGetResponseDataRelationshipsAuthor,
        )
        from ..models.workitems_single_get_response_data_relationships_backlinked_work_items import (
            WorkitemsSingleGetResponseDataRelationshipsBacklinkedWorkItems,
        )
        from ..models.workitems_single_get_response_data_relationships_categories import (
            WorkitemsSingleGetResponseDataRelationshipsCategories,
        )
        from ..models.workitems_single_get_response_data_relationships_comments import (
            WorkitemsSingleGetResponseDataRelationshipsComments,
        )
        from ..models.workitems_single_get_response_data_relationships_externally_linked_work_items import (
            WorkitemsSingleGetResponseDataRelationshipsExternallyLinkedWorkItems,
        )
        from ..models.workitems_single_get_response_data_relationships_linked_oslc_resources import (
            WorkitemsSingleGetResponseDataRelationshipsLinkedOslcResources,
        )
        from ..models.workitems_single_get_response_data_relationships_linked_revisions import (
            WorkitemsSingleGetResponseDataRelationshipsLinkedRevisions,
        )
        from ..models.workitems_single_get_response_data_relationships_linked_work_items import (
            WorkitemsSingleGetResponseDataRelationshipsLinkedWorkItems,
        )
        from ..models.workitems_single_get_response_data_relationships_module import (
            WorkitemsSingleGetResponseDataRelationshipsModule,
        )
        from ..models.workitems_single_get_response_data_relationships_planned_in import (
            WorkitemsSingleGetResponseDataRelationshipsPlannedIn,
        )
        from ..models.workitems_single_get_response_data_relationships_project import (
            WorkitemsSingleGetResponseDataRelationshipsProject,
        )
        from ..models.workitems_single_get_response_data_relationships_test_steps import (
            WorkitemsSingleGetResponseDataRelationshipsTestSteps,
        )
        from ..models.workitems_single_get_response_data_relationships_votes import (
            WorkitemsSingleGetResponseDataRelationshipsVotes,
        )
        from ..models.workitems_single_get_response_data_relationships_watches import (
            WorkitemsSingleGetResponseDataRelationshipsWatches,
        )
        from ..models.workitems_single_get_response_data_relationships_work_records import (
            WorkitemsSingleGetResponseDataRelationshipsWorkRecords,
        )

        d = src_dict.copy()
        _approvals = d.pop("approvals", UNSET)
        approvals: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsApprovals
        ]
        if isinstance(_approvals, Unset):
            approvals = UNSET
        else:
            approvals = (
                WorkitemsSingleGetResponseDataRelationshipsApprovals.from_dict(
                    _approvals
                )
            )

        _assignee = d.pop("assignee", UNSET)
        assignee: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsAssignee
        ]
        if isinstance(_assignee, Unset):
            assignee = UNSET
        else:
            assignee = (
                WorkitemsSingleGetResponseDataRelationshipsAssignee.from_dict(
                    _assignee
                )
            )

        _attachments = d.pop("attachments", UNSET)
        attachments: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsAttachments
        ]
        if isinstance(_attachments, Unset):
            attachments = UNSET
        else:
            attachments = WorkitemsSingleGetResponseDataRelationshipsAttachments.from_dict(
                _attachments
            )

        _author = d.pop("author", UNSET)
        author: Union[Unset, WorkitemsSingleGetResponseDataRelationshipsAuthor]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = (
                WorkitemsSingleGetResponseDataRelationshipsAuthor.from_dict(
                    _author
                )
            )

        _backlinked_work_items = d.pop("backlinkedWorkItems", UNSET)
        backlinked_work_items: Union[
            Unset,
            WorkitemsSingleGetResponseDataRelationshipsBacklinkedWorkItems,
        ]
        if isinstance(_backlinked_work_items, Unset):
            backlinked_work_items = UNSET
        else:
            backlinked_work_items = WorkitemsSingleGetResponseDataRelationshipsBacklinkedWorkItems.from_dict(
                _backlinked_work_items
            )

        _categories = d.pop("categories", UNSET)
        categories: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsCategories
        ]
        if isinstance(_categories, Unset):
            categories = UNSET
        else:
            categories = WorkitemsSingleGetResponseDataRelationshipsCategories.from_dict(
                _categories
            )

        _comments = d.pop("comments", UNSET)
        comments: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsComments
        ]
        if isinstance(_comments, Unset):
            comments = UNSET
        else:
            comments = (
                WorkitemsSingleGetResponseDataRelationshipsComments.from_dict(
                    _comments
                )
            )

        _externally_linked_work_items = d.pop(
            "externallyLinkedWorkItems", UNSET
        )
        externally_linked_work_items: Union[
            Unset,
            WorkitemsSingleGetResponseDataRelationshipsExternallyLinkedWorkItems,
        ]
        if isinstance(_externally_linked_work_items, Unset):
            externally_linked_work_items = UNSET
        else:
            externally_linked_work_items = WorkitemsSingleGetResponseDataRelationshipsExternallyLinkedWorkItems.from_dict(
                _externally_linked_work_items
            )

        _linked_oslc_resources = d.pop("linkedOslcResources", UNSET)
        linked_oslc_resources: Union[
            Unset,
            WorkitemsSingleGetResponseDataRelationshipsLinkedOslcResources,
        ]
        if isinstance(_linked_oslc_resources, Unset):
            linked_oslc_resources = UNSET
        else:
            linked_oslc_resources = WorkitemsSingleGetResponseDataRelationshipsLinkedOslcResources.from_dict(
                _linked_oslc_resources
            )

        _linked_revisions = d.pop("linkedRevisions", UNSET)
        linked_revisions: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsLinkedRevisions
        ]
        if isinstance(_linked_revisions, Unset):
            linked_revisions = UNSET
        else:
            linked_revisions = WorkitemsSingleGetResponseDataRelationshipsLinkedRevisions.from_dict(
                _linked_revisions
            )

        _linked_work_items = d.pop("linkedWorkItems", UNSET)
        linked_work_items: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsLinkedWorkItems
        ]
        if isinstance(_linked_work_items, Unset):
            linked_work_items = UNSET
        else:
            linked_work_items = WorkitemsSingleGetResponseDataRelationshipsLinkedWorkItems.from_dict(
                _linked_work_items
            )

        _module = d.pop("module", UNSET)
        module: Union[Unset, WorkitemsSingleGetResponseDataRelationshipsModule]
        if isinstance(_module, Unset):
            module = UNSET
        else:
            module = (
                WorkitemsSingleGetResponseDataRelationshipsModule.from_dict(
                    _module
                )
            )

        _planned_in = d.pop("plannedIn", UNSET)
        planned_in: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsPlannedIn
        ]
        if isinstance(_planned_in, Unset):
            planned_in = UNSET
        else:
            planned_in = (
                WorkitemsSingleGetResponseDataRelationshipsPlannedIn.from_dict(
                    _planned_in
                )
            )

        _project = d.pop("project", UNSET)
        project: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsProject
        ]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = (
                WorkitemsSingleGetResponseDataRelationshipsProject.from_dict(
                    _project
                )
            )

        _test_steps = d.pop("testSteps", UNSET)
        test_steps: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsTestSteps
        ]
        if isinstance(_test_steps, Unset):
            test_steps = UNSET
        else:
            test_steps = (
                WorkitemsSingleGetResponseDataRelationshipsTestSteps.from_dict(
                    _test_steps
                )
            )

        _votes = d.pop("votes", UNSET)
        votes: Union[Unset, WorkitemsSingleGetResponseDataRelationshipsVotes]
        if isinstance(_votes, Unset):
            votes = UNSET
        else:
            votes = WorkitemsSingleGetResponseDataRelationshipsVotes.from_dict(
                _votes
            )

        _watches = d.pop("watches", UNSET)
        watches: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsWatches
        ]
        if isinstance(_watches, Unset):
            watches = UNSET
        else:
            watches = (
                WorkitemsSingleGetResponseDataRelationshipsWatches.from_dict(
                    _watches
                )
            )

        _work_records = d.pop("workRecords", UNSET)
        work_records: Union[
            Unset, WorkitemsSingleGetResponseDataRelationshipsWorkRecords
        ]
        if isinstance(_work_records, Unset):
            work_records = UNSET
        else:
            work_records = WorkitemsSingleGetResponseDataRelationshipsWorkRecords.from_dict(
                _work_records
            )

        workitems_single_get_response_data_relationships_obj = cls(
            approvals=approvals,
            assignee=assignee,
            attachments=attachments,
            author=author,
            backlinked_work_items=backlinked_work_items,
            categories=categories,
            comments=comments,
            externally_linked_work_items=externally_linked_work_items,
            linked_oslc_resources=linked_oslc_resources,
            linked_revisions=linked_revisions,
            linked_work_items=linked_work_items,
            module=module,
            planned_in=planned_in,
            project=project,
            test_steps=test_steps,
            votes=votes,
            watches=watches,
            work_records=work_records,
        )

        workitems_single_get_response_data_relationships_obj.additional_properties = (
            d
        )
        return workitems_single_get_response_data_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.document_parts_list_post_response_data_item_type import (
    DocumentPartsListPostResponseDataItemType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.document_parts_list_post_response_data_item_links import (
        DocumentPartsListPostResponseDataItemLinks,
    )


T = TypeVar("T", bound="DocumentPartsListPostResponseDataItem")


@_attrs_define
class DocumentPartsListPostResponseDataItem:
    """
    Attributes:
        type (Union[Unset, DocumentPartsListPostResponseDataItemType]):
        id (Union[Unset, str]):  Example: MyProjectId/MySpaceId/MyDocumentId/workitem_MyWorkItemId.
        links (Union[Unset, DocumentPartsListPostResponseDataItemLinks]):
    """

    type: Union[Unset, DocumentPartsListPostResponseDataItemType] = UNSET
    id: Union[Unset, str] = UNSET
    links: Union[Unset, "DocumentPartsListPostResponseDataItemLinks"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        id = self.id

        links: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.links, Unset):
            links = self.links.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if id is not UNSET:
            field_dict["id"] = id
        if links is not UNSET:
            field_dict["links"] = links

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.document_parts_list_post_response_data_item_links import (
            DocumentPartsListPostResponseDataItemLinks,
        )

        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, DocumentPartsListPostResponseDataItemType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = DocumentPartsListPostResponseDataItemType(_type)

        id = d.pop("id", UNSET)

        _links = d.pop("links", UNSET)
        links: Union[Unset, DocumentPartsListPostResponseDataItemLinks]
        if isinstance(_links, Unset):
            links = UNSET
        else:
            links = DocumentPartsListPostResponseDataItemLinks.from_dict(
                _links
            )

        document_parts_list_post_response_data_item_obj = cls(
            type=type,
            id=id,
            links=links,
        )

        document_parts_list_post_response_data_item_obj.additional_properties = (
            d
        )
        return document_parts_list_post_response_data_item_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DocumentPartsSingleGetResponseDataAttributes")


@_attrs_define
class DocumentPartsSingleGetResponseDataAttributes:
    """
    Attributes:
        content (Union[Unset, str]):  Example: <div id="polarion_wiki macro name=module-
            workitem;params=id=workitem_MyWorkItemId"></div>.
        external (Union[Unset, bool]):
        id (Union[Unset, str]):  Example: workitem_MyWorkItemId.
        level (Union[Unset, int]):
        type (Union[Unset, str]):  Example: workitem.
    """

    content: Union[Unset, str] = UNSET
    external: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    level: Union[Unset, int] = UNSET
    type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        content = self.content

        external = self.external

        id = self.id

        level = self.level

        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if content is not UNSET:
            field_dict["content"] = content
        if external is not UNSET:
            field_dict["external"] = external
        if id is not UNSET:
            field_dict["id"] = id
        if level is not UNSET:
            field_dict["level"] = level
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        content = d.pop("content", UNSET)

        external = d.pop("external", UNSET)

        id = d.pop("id", UNSET)

        level = d.pop("level", UNSET)

        type = d.pop("type", UNSET)

        document_parts_single_get_response_data_attributes_obj = cls(
            content=content,
            external=external,
            id=id,
            level=level,
            type=type,
        )

        document_parts_single_get_response_data_attributes_obj.additional_properties = (
            d
        )
        return document_parts_single_get_response_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.enumerations_single_get_response_data_attributes_options_item import (
        EnumerationsSingleGetResponseDataAttributesOptionsItem,
    )


T = TypeVar("T", bound="EnumerationsSingleGetResponseDataAttributes")


@_attrs_define
class EnumerationsSingleGetResponseDataAttributes:
    """
    Attributes:
        enum_context (Union[Unset, str]):  Example: id.
        enum_name (Union[Unset, str]):  Example: id.
        options (Union[Unset, List['EnumerationsSingleGetResponseDataAttributesOptionsItem']]):
        target_type (Union[Unset, str]):  Example: id.
    """

    enum_context: Union[Unset, str] = UNSET
    enum_name: Union[Unset, str] = UNSET
    options: Union[
        Unset, List["EnumerationsSingleGetResponseDataAttributesOptionsItem"]
    ] = UNSET
    target_type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        enum_context = self.enum_context

        enum_name = self.enum_name

        options: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.options, Unset):
            options = []
            for options_item_data in self.options:
                options_item = options_item_data.to_dict()
                options.append(options_item)

        target_type = self.target_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if enum_context is not UNSET:
            field_dict["enumContext"] = enum_context
        if enum_name is not UNSET:
            field_dict["enumName"] = enum_name
        if options is not UNSET:
            field_dict["options"] = options
        if target_type is not UNSET:
            field_dict["targetType"] = target_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.enumerations_single_get_response_data_attributes_options_item import (
            EnumerationsSingleGetResponseDataAttributesOptionsItem,
        )

        d = src_dict.copy()
        enum_context = d.pop("enumContext", UNSET)

        enum_name = d.pop("enumName", UNSET)

        options = []
        _options = d.pop("options", UNSET)
        for options_item_data in _options or []:
            options_item = EnumerationsSingleGetResponseDataAttributesOptionsItem.from_dict(
                options_item_data
            )

            options.append(options_item)

        target_type = d.pop("targetType", UNSET)

        enumerations_single_get_response_data_attributes_obj = cls(
            enum_context=enum_context,
            enum_name=enum_name,
            options=options,
            target_type=target_type,
        )

        enumerations_single_get_response_data_attributes_obj.additional_properties = (
            d
        )
        return enumerations_single_get_response_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workitems_single_patch_request_data_attributes_description import (
        WorkitemsSinglePatchRequestDataAttributesDescription,
    )
    from ..models.workitems_single_patch_request_data_attributes_hyperlinks_item import (
        WorkitemsSinglePatchRequestDataAttributesHyperlinksItem,
    )


T = TypeVar("T", bound="WorkitemsSinglePatchRequestDataAttributes")


@_attrs_define
class WorkitemsSinglePatchRequestDataAttributes:
    """
    Attributes:
        description (Union[Unset, WorkitemsSinglePatchRequestDataAttributesDescription]):
        due_date (Union[Unset, datetime.date]):  Example: 1970-01-01.
        hyperlinks (Union[Unset, List['WorkitemsSinglePatchRequestDataAttributesHyperlinksItem']]):
        initial_estimate (Union[Unset, str]):  Example: 5 1/2d.
        priority (Union[Unset, str]):  Example: 90.0.
        remaining_estimate (Union[Unset, str]):  Example: 5 1/2d.
        resolution (Union[Unset, str]):  Example: done.
        resolved_on (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        severity (Union[Unset, str]):  Example: blocker.
        status (Union[Unset, str]):  Example: open.
        time_spent (Union[Unset, str]):  Example: 5 1/2d.
        title (Union[Unset, str]):  Example: Title.
    """

    description: Union[
        Unset, "WorkitemsSinglePatchRequestDataAttributesDescription"
    ] = UNSET
    due_date: Union[Unset, datetime.date] = UNSET
    hyperlinks: Union[
        Unset, List["WorkitemsSinglePatchRequestDataAttributesHyperlinksItem"]
    ] = UNSET
    initial_estimate: Union[Unset, str] = UNSET
    priority: Union[Unset, str] = UNSET
    remaining_estimate: Union[Unset, str] = UNSET
    resolution: Union[Unset, str] = UNSET
    resolved_on: Union[Unset, datetime.datetime] = UNSET
    severity: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    time_spent: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        description: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description.to_dict()

        due_date: Union[Unset, str] = UNSET
        if not isinstance(self.due_date, Unset):
            due_date = self.due_date.isoformat()

        hyperlinks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.hyperlinks, Unset):
            hyperlinks = []
            for hyperlinks_item_data in self.hyperlinks:
                hyperlinks_item = hyperlinks_item_data.to_dict()
                hyperlinks.append(hyperlinks_item)

        initial_estimate = self.initial_estimate

        priority = self.priority

        remaining_estimate = self.remaining_estimate

        resolution = self.resolution

        resolved_on: Union[Unset, str] = UNSET
        if not isinstance(self.resolved_on, Unset):
            resolved_on = self.resolved_on.isoformat()

        severity = self.severity

        status = self.status

        time_spent = self.time_spent

        title = self.title

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if due_date is not UNSET:
            field_dict["dueDate"] = due_date
        if hyperlinks is not UNSET:
            field_dict["hyperlinks"] = hyperlinks
        if initial_estimate is not UNSET:
            field_dict["initialEstimate"] = initial_estimate
        if priority is not UNSET:
            field_dict["priority"] = priority
        if remaining_estimate is not UNSET:
            field_dict["remainingEstimate"] = remaining_estimate
        if resolution is not UNSET:
            field_dict["resolution"] = resolution
        if resolved_on is not UNSET:
            field_dict["resolvedOn"] = resolved_on
        if severity is not UNSET:
            field_dict["severity"] = severity
        if status is not UNSET:
            field_dict["status"] = status
        if time_spent is not UNSET:
            field_dict["timeSpent"] = time_spent
        if title is not UNSET:
            field_dict["title"] = title

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workitems_single_patch_request_data_attributes_description import (
            WorkitemsSinglePatchRequestDataAttributesDescription,
        )
        from ..models.workitems_single_patch_request_data_attributes_hyperlinks_item import (
            WorkitemsSinglePatchRequestDataAttributesHyperlinksItem,
        )

        d = src_dict.copy()
        _description = d.pop("description", UNSET)
        description: Union[
            Unset, WorkitemsSinglePatchRequestDataAttributesDescription
        ]
        if isinstance(_description, Unset):
            description = UNSET
        else:
            description = (
                WorkitemsSinglePatchRequestDataAttributesDescription.from_dict(
                    _description
                )
            )

        _due_date = d.pop("dueDate", UNSET)
        due_date: Union[Unset, datetime.date]
        if isinstance(_due_date, Unset):
            due_date = UNSET
        else:
            due_date = isoparse(_due_date).date()

        hyperlinks = []
        _hyperlinks = d.pop("hyperlinks", UNSET)
        for hyperlinks_item_data in _hyperlinks or []:
            hyperlinks_item = WorkitemsSinglePatchRequestDataAttributesHyperlinksItem.from_dict(
                hyperlinks_item_data
            )

            hyperlinks.append(hyperlinks_item)

        initial_estimate = d.pop("initialEstimate", UNSET)

        priority = d.pop("priority", UNSET)

        remaining_estimate = d.pop("remainingEstimate", UNSET)

        resolution = d.pop("resolution", UNSET)

        _resolved_on = d.pop("resolvedOn", UNSET)
        resolved_on: Union[Unset, datetime.datetime]
        if isinstance(_resolved_on, Unset):
            resolved_on = UNSET
        else:
            resolved_on = isoparse(_resolved_on)

        severity = d.pop("severity", UNSET)

        status = d.pop("status", UNSET)

        time_spent = d.pop("timeSpent", UNSET)

        title = d.pop("title", UNSET)

        workitems_single_patch_request_data_attributes_obj = cls(
            description=description,
            due_date=due_date,
            hyperlinks=hyperlinks,
            initial_estimate=initial_estimate,
            priority=priority,
            remaining_estimate=remaining_estimate,
            resolution=resolution,
            resolved_on=resolved_on,
            severity=severity,
            status=status,
            time_spent=time_spent,
            title=title,
        )

        workitems_single_patch_request_data_attributes_obj.additional_properties = (
            d
        )
        return workitems_single_patch_request_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Implementation of LinkedWorkItems operations."""
import itertools
import typing as t

from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.open_api_client import models as api_models
from polarion_rest_api_client.open_api_client import types as oa_types
from polarion_rest_api_client.open_api_client.api.linked_work_items import (
    delete_linked_work_items,
    get_linked_work_items,
    post_linked_work_items,
)

from . import base_classes as bc


class WorkItemLinks(bc.ItemsClient[dm.WorkItemLink]):
    """A client providing LinkedWorkItems functions."""

    def get(self, *args, **kwargs) -> dm.WorkItemLink:
        """Return a specific link - not implemented yet."""
        raise NotImplementedError

    def get_multi(  # type: ignore[override]
        self,
        work_item_id: str,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: dict[str, str] | None = None,
        include: str | None | oa_types.Unset = None,
    ) -> tuple[list[dm.WorkItemLink], bool]:
        """Get the work item links for the given work item on a page.

        In addition, a flag whether a next page is available is
        returned. Define a fields dictionary as described in the
        Polarion API documentation to get certain fields.
        """
        if fields is None:
            fields = self._client.default_fields.linkedworkitems

        if include is None:
            include = oa_types.UNSET

        sparse_fields = self._build_sparse_fields(fields)
        response = get_linked_work_items.sync_detailed(
            self._project_id,
            work_item_id,
            client=self._client.client,
            fields=sparse_fields,
            include=include,
            pagesize=page_size,
            pagenumber=page_number,
        )

        self._raise_on_error(response)

        linked_work_item_response = response.parsed

        work_item_links: list[dm.WorkItemLink] = []
        next_page = False
        if (
            isinstance(
                linked_work_item_response,
                api_models.LinkedworkitemsListGetResponse,
            )
            and linked_work_item_response.data
        ):
            for link in linked_work_item_response.data:
                assert isinstance(link.id, str)
                assert isinstance(
                    link.attributes,
                    api_models.LinkedworkitemsListGetResponseDataItemAttributes,  # pylint: disable=line-too-long
                )

                work_item_links.append(
                    self._parse_work_item_link(
                        link.id, link.attributes.suspect, work_item_id
                    )
                )

            next_page = isinstance(
                linked_work_item_response.links,
                api_models.LinkedworkitemsListGetResponseLinks,
            ) and bool(linked_work_item_response.links.next_)

        return work_item_links, next_page

    def _parse_work_item_link(self, link_id, suspect, work_item_id):
        info = link_id.split("/")
        assert len(info) == 5
        role_id, target_project_id, linked_work_item_id = info[2:]
        if isinstance(suspect, oa_types.Unset):
            suspect = False
        work_item_link = dm.WorkItemLink(
            work_item_id,
            linked_work_item_id,
            role_id,
            suspect,
            target_project_id,
        )
        return work_item_link

    def _split_into_batches(
        self, items: list[dm.WorkItemLink]
    ) -> t.Generator[list[dm.WorkItemLink], None, None]:
        for _, group in itertools.groupby(
            items, lambda x: x.primary_work_item_id
        ):
            yield from super()._split_into_batches(list(group))

    def _create(self, items: list[dm.WorkItemLink]):
        response = post_linked_work_items.sync_detailed(
            self._project_id,
            items[0].primary_work_item_id,
            client=self._client.client,
            # pylint: disable=line-too-long
            body=api_models.LinkedworkitemsListPostRequest(
                data=[
                    api_models.LinkedworkitemsListPostRequestDataItem(
                        type=api_models.LinkedworkitemsListPostRequestDataItemType.LINKEDWORKITEMS,
                        attributes=api_models.LinkedworkitemsListPostRequestDataItemAttributes(
                            role=work_item_link.role,
                            suspect=work_item_link.suspect or False,
                        ),
                        relationships=api_models.LinkedworkitemsListPostRequestDataItemRelationships(
                            work_item=api_models.LinkedworkitemsListPostRequestDataItemRelationshipsWorkItem(
                                data=api_models.LinkedworkitemsListPostRequestDataItemRelationshipsWorkItemData(
                                    type=api_models.LinkedworkitemsListPostRequestDataItemRelationshipsWorkItemDataType.WORKITEMS,
                                    id=f"{work_item_link.secondary_work_item_project or self._project_id}/{work_item_link.secondary_work_item_id}",
                                )
                            )
                        ),
                    )
                    for work_item_link in items
                ]
            ),
            # pylint: enable=line-too-long
        )

        self._raise_on_error(response)

    def _delete(self, items: list[dm.WorkItemLink]):
        response = delete_linked_work_items.sync_detailed(
            self._project_id,
            items[0].primary_work_item_id,
            client=self._client.client,
            # pylint: disable=line-too-long
            body=api_models.LinkedworkitemsListDeleteRequest(
                data=[
                    api_models.LinkedworkitemsListDeleteRequestDataItem(
                        type=api_models.LinkedworkitemsListDeleteRequestDataItemType.LINKEDWORKITEMS,
                        id=f"{self._project_id}/{work_item_link.primary_work_item_id}/{work_item_link.role}/{work_item_link.secondary_work_item_project or self._project_id}/{work_item_link.secondary_work_item_id}",
                    )
                    for work_item_link in items
                ]
            ),
            # pylint: enable=line-too-long
        )
        self._raise_on_error(response)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.pages_single_get_response_data_relationships_attachments import (
        PagesSingleGetResponseDataRelationshipsAttachments,
    )
    from ..models.pages_single_get_response_data_relationships_author import (
        PagesSingleGetResponseDataRelationshipsAuthor,
    )
    from ..models.pages_single_get_response_data_relationships_project import (
        PagesSingleGetResponseDataRelationshipsProject,
    )
    from ..models.pages_single_get_response_data_relationships_updated_by import (
        PagesSingleGetResponseDataRelationshipsUpdatedBy,
    )


T = TypeVar("T", bound="PagesSingleGetResponseDataRelationships")


@_attrs_define
class PagesSingleGetResponseDataRelationships:
    """
    Attributes:
        attachments (Union[Unset, PagesSingleGetResponseDataRelationshipsAttachments]):
        author (Union[Unset, PagesSingleGetResponseDataRelationshipsAuthor]):
        project (Union[Unset, PagesSingleGetResponseDataRelationshipsProject]):
        updated_by (Union[Unset, PagesSingleGetResponseDataRelationshipsUpdatedBy]):
    """

    attachments: Union[
        Unset, "PagesSingleGetResponseDataRelationshipsAttachments"
    ] = UNSET
    author: Union[Unset, "PagesSingleGetResponseDataRelationshipsAuthor"] = (
        UNSET
    )
    project: Union[Unset, "PagesSingleGetResponseDataRelationshipsProject"] = (
        UNSET
    )
    updated_by: Union[
        Unset, "PagesSingleGetResponseDataRelationshipsUpdatedBy"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        attachments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = self.attachments.to_dict()

        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        updated_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.updated_by, Unset):
            updated_by = self.updated_by.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if author is not UNSET:
            field_dict["author"] = author
        if project is not UNSET:
            field_dict["project"] = project
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.pages_single_get_response_data_relationships_attachments import (
            PagesSingleGetResponseDataRelationshipsAttachments,
        )
        from ..models.pages_single_get_response_data_relationships_author import (
            PagesSingleGetResponseDataRelationshipsAuthor,
        )
        from ..models.pages_single_get_response_data_relationships_project import (
            PagesSingleGetResponseDataRelationshipsProject,
        )
        from ..models.pages_single_get_response_data_relationships_updated_by import (
            PagesSingleGetResponseDataRelationshipsUpdatedBy,
        )

        d = src_dict.copy()
        _attachments = d.pop("attachments", UNSET)
        attachments: Union[
            Unset, PagesSingleGetResponseDataRelationshipsAttachments
        ]
        if isinstance(_attachments, Unset):
            attachments = UNSET
        else:
            attachments = (
                PagesSingleGetResponseDataRelationshipsAttachments.from_dict(
                    _attachments
                )
            )

        _author = d.pop("author", UNSET)
        author: Union[Unset, PagesSingleGetResponseDataRelationshipsAuthor]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = PagesSingleGetResponseDataRelationshipsAuthor.from_dict(
                _author
            )

        _project = d.pop("project", UNSET)
        project: Union[Unset, PagesSingleGetResponseDataRelationshipsProject]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = PagesSingleGetResponseDataRelationshipsProject.from_dict(
                _project
            )

        _updated_by = d.pop("updatedBy", UNSET)
        updated_by: Union[
            Unset, PagesSingleGetResponseDataRelationshipsUpdatedBy
        ]
        if isinstance(_updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = (
                PagesSingleGetResponseDataRelationshipsUpdatedBy.from_dict(
                    _updated_by
                )
            )

        pages_single_get_response_data_relationships_obj = cls(
            attachments=attachments,
            author=author,
            project=project,
            updated_by=updated_by,
        )

        pages_single_get_response_data_relationships_obj.additional_properties = (
            d
        )
        return pages_single_get_response_data_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.document_attachments_list_post_request_data_item_type import (
    DocumentAttachmentsListPostRequestDataItemType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.document_attachments_list_post_request_data_item_attributes import (
        DocumentAttachmentsListPostRequestDataItemAttributes,
    )


T = TypeVar("T", bound="DocumentAttachmentsListPostRequestDataItem")


@_attrs_define
class DocumentAttachmentsListPostRequestDataItem:
    """
    Attributes:
        type (Union[Unset, DocumentAttachmentsListPostRequestDataItemType]):
        attributes (Union[Unset, DocumentAttachmentsListPostRequestDataItemAttributes]):
        lid (Union[Unset, str]):
    """

    type: Union[Unset, DocumentAttachmentsListPostRequestDataItemType] = UNSET
    attributes: Union[
        Unset, "DocumentAttachmentsListPostRequestDataItemAttributes"
    ] = UNSET
    lid: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        attributes: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attributes, Unset):
            attributes = self.attributes.to_dict()

        lid = self.lid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if attributes is not UNSET:
            field_dict["attributes"] = attributes
        if lid is not UNSET:
            field_dict["lid"] = lid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.document_attachments_list_post_request_data_item_attributes import (
            DocumentAttachmentsListPostRequestDataItemAttributes,
        )

        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, DocumentAttachmentsListPostRequestDataItemType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = DocumentAttachmentsListPostRequestDataItemType(_type)

        _attributes = d.pop("attributes", UNSET)
        attributes: Union[
            Unset, DocumentAttachmentsListPostRequestDataItemAttributes
        ]
        if isinstance(_attributes, Unset):
            attributes = UNSET
        else:
            attributes = (
                DocumentAttachmentsListPostRequestDataItemAttributes.from_dict(
                    _attributes
                )
            )

        lid = d.pop("lid", UNSET)

        document_attachments_list_post_request_data_item_obj = cls(
            type=type,
            attributes=attributes,
            lid=lid,
        )

        document_attachments_list_post_request_data_item_obj.additional_properties = (
            d
        )
        return document_attachments_list_post_request_data_item_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.testrecords_list_get_response_data_item_meta_errors_item import (
        TestrecordsListGetResponseDataItemMetaErrorsItem,
    )


T = TypeVar("T", bound="TestrecordsListGetResponseDataItemMeta")


@_attrs_define
class TestrecordsListGetResponseDataItemMeta:
    """
    Attributes:
        errors (Union[Unset, List['TestrecordsListGetResponseDataItemMetaErrorsItem']]):
    """

    errors: Union[
        Unset, List["TestrecordsListGetResponseDataItemMetaErrorsItem"]
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        errors: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.errors, Unset):
            errors = []
            for errors_item_data in self.errors:
                errors_item = errors_item_data.to_dict()
                errors.append(errors_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if errors is not UNSET:
            field_dict["errors"] = errors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.testrecords_list_get_response_data_item_meta_errors_item import (
            TestrecordsListGetResponseDataItemMetaErrorsItem,
        )

        d = src_dict.copy()
        errors = []
        _errors = d.pop("errors", UNSET)
        for errors_item_data in _errors or []:
            errors_item = (
                TestrecordsListGetResponseDataItemMetaErrorsItem.from_dict(
                    errors_item_data
                )
            )

            errors.append(errors_item)

        testrecords_list_get_response_data_item_meta_obj = cls(
            errors=errors,
        )

        testrecords_list_get_response_data_item_meta_obj.additional_properties = (
            d
        )
        return testrecords_list_get_response_data_item_meta_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""A client for a specific project, using the session of PolarionClient."""
import typing as t

from polarion_rest_api_client.clients import base_classes as bc
from polarion_rest_api_client.open_api_client.api.projects import get_project

from . import documents, test_runs, work_items

if t.TYPE_CHECKING:
    from polarion_rest_api_client import client as polarion_client


class ProjectClient(bc.BaseClient):
    """A client for a specific project."""

    def __init__(
        self,
        project_id: str,
        client: "polarion_client.PolarionClient",
        delete_status: str | None = None,
    ):
        super().__init__(project_id, client)

        self.work_items = work_items.WorkItems(
            project_id, client, delete_status
        )
        self.test_runs = test_runs.TestRuns(project_id, client)
        self.documents = documents.Documents(project_id, client)

    def exists(self):
        """Return True, if the clients project exists."""
        response = get_project.sync_detailed(
            self._project_id, client=self._client.client
        )
        if response.status_code == 200:
            return True
        return False



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkitemAttachmentsListGetResponseDataItemAttributes")


@_attrs_define
class WorkitemAttachmentsListGetResponseDataItemAttributes:
    """
    Attributes:
        file_name (Union[Unset, str]):  Example: File Name.
        id (Union[Unset, str]):  Example: MyAttachmentId.
        length (Union[Unset, int]):
        title (Union[Unset, str]):  Example: Title.
        updated (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
    """

    file_name: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    length: Union[Unset, int] = UNSET
    title: Union[Unset, str] = UNSET
    updated: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        file_name = self.file_name

        id = self.id

        length = self.length

        title = self.title

        updated: Union[Unset, str] = UNSET
        if not isinstance(self.updated, Unset):
            updated = self.updated.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if file_name is not UNSET:
            field_dict["fileName"] = file_name
        if id is not UNSET:
            field_dict["id"] = id
        if length is not UNSET:
            field_dict["length"] = length
        if title is not UNSET:
            field_dict["title"] = title
        if updated is not UNSET:
            field_dict["updated"] = updated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file_name = d.pop("fileName", UNSET)

        id = d.pop("id", UNSET)

        length = d.pop("length", UNSET)

        title = d.pop("title", UNSET)

        _updated = d.pop("updated", UNSET)
        updated: Union[Unset, datetime.datetime]
        if isinstance(_updated, Unset):
            updated = UNSET
        else:
            updated = isoparse(_updated)

        workitem_attachments_list_get_response_data_item_attributes_obj = cls(
            file_name=file_name,
            id=id,
            length=length,
            title=title,
            updated=updated,
        )

        workitem_attachments_list_get_response_data_item_attributes_obj.additional_properties = (
            d
        )
        return workitem_attachments_list_get_response_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.usergroups_single_get_response_data_attributes_description import (
        UsergroupsSingleGetResponseDataAttributesDescription,
    )


T = TypeVar("T", bound="UsergroupsSingleGetResponseDataAttributes")


@_attrs_define
class UsergroupsSingleGetResponseDataAttributes:
    """
    Attributes:
        description (Union[Unset, UsergroupsSingleGetResponseDataAttributesDescription]):
        id (Union[Unset, str]):  Example: MyUserGroupId.
        ldap_search_filter (Union[Unset, str]):  Example: LDAP Search Filter.
        name (Union[Unset, str]):  Example: Name.
        sso_synchronization_allowed (Union[Unset, bool]):
    """

    description: Union[
        Unset, "UsergroupsSingleGetResponseDataAttributesDescription"
    ] = UNSET
    id: Union[Unset, str] = UNSET
    ldap_search_filter: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    sso_synchronization_allowed: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        description: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description.to_dict()

        id = self.id

        ldap_search_filter = self.ldap_search_filter

        name = self.name

        sso_synchronization_allowed = self.sso_synchronization_allowed

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if id is not UNSET:
            field_dict["id"] = id
        if ldap_search_filter is not UNSET:
            field_dict["ldapSearchFilter"] = ldap_search_filter
        if name is not UNSET:
            field_dict["name"] = name
        if sso_synchronization_allowed is not UNSET:
            field_dict["ssoSynchronizationAllowed"] = (
                sso_synchronization_allowed
            )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.usergroups_single_get_response_data_attributes_description import (
            UsergroupsSingleGetResponseDataAttributesDescription,
        )

        d = src_dict.copy()
        _description = d.pop("description", UNSET)
        description: Union[
            Unset, UsergroupsSingleGetResponseDataAttributesDescription
        ]
        if isinstance(_description, Unset):
            description = UNSET
        else:
            description = (
                UsergroupsSingleGetResponseDataAttributesDescription.from_dict(
                    _description
                )
            )

        id = d.pop("id", UNSET)

        ldap_search_filter = d.pop("ldapSearchFilter", UNSET)

        name = d.pop("name", UNSET)

        sso_synchronization_allowed = d.pop("ssoSynchronizationAllowed", UNSET)

        usergroups_single_get_response_data_attributes_obj = cls(
            description=description,
            id=id,
            ldap_search_filter=ldap_search_filter,
            name=name,
            sso_synchronization_allowed=sso_synchronization_allowed,
        )

        usergroups_single_get_response_data_attributes_obj.additional_properties = (
            d
        )
        return usergroups_single_get_response_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.branch_documents_request_body_document_configurations_item import (
        BranchDocumentsRequestBodyDocumentConfigurationsItem,
    )


T = TypeVar("T", bound="BranchDocumentsRequestBody")


@_attrs_define
class BranchDocumentsRequestBody:
    """
    Attributes:
        document_configurations (List['BranchDocumentsRequestBodyDocumentConfigurationsItem']):
    """

    document_configurations: List[
        "BranchDocumentsRequestBodyDocumentConfigurationsItem"
    ]
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        document_configurations = []
        for document_configurations_item_data in self.document_configurations:
            document_configurations_item = (
                document_configurations_item_data.to_dict()
            )
            document_configurations.append(document_configurations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "documentConfigurations": document_configurations,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.branch_documents_request_body_document_configurations_item import (
            BranchDocumentsRequestBodyDocumentConfigurationsItem,
        )

        d = src_dict.copy()
        document_configurations = []
        _document_configurations = d.pop("documentConfigurations")
        for document_configurations_item_data in _document_configurations:
            document_configurations_item = (
                BranchDocumentsRequestBodyDocumentConfigurationsItem.from_dict(
                    document_configurations_item_data
                )
            )

            document_configurations.append(document_configurations_item)

        branch_documents_request_body_obj = cls(
            document_configurations=document_configurations,
        )

        branch_documents_request_body_obj.additional_properties = d
        return branch_documents_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
